# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from typing import Any, Dict, Optional, Self

from pydantic import BaseModel, ConfigDict, Field, StrictInt

from algoliasearch.insights.models.discount import Discount
from algoliasearch.insights.models.price import Price


class ObjectData(BaseModel):
    """
    ObjectData
    """

    price: Optional[Price] = None
    quantity: Optional[StrictInt] = Field(
        default=None,
        description="Quantity of a product that has been purchased or added to the cart. The total purchase value is the sum of `quantity` multiplied with the `price` for each purchased item. ",
    )
    discount: Optional[Discount] = None

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ObjectData from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        if self.price:
            _dict["price"] = self.price.to_dict()
        if self.discount:
            _dict["discount"] = self.discount.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ObjectData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "price": (
                    Price.from_dict(obj.get("price"))
                    if obj.get("price") is not None
                    else None
                ),
                "quantity": obj.get("quantity"),
                "discount": (
                    Discount.from_dict(obj.get("discount"))
                    if obj.get("discount") is not None
                    else None
                ),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from typing import Annotated, Any, Dict, List, Optional, Self

from pydantic import BaseModel, ConfigDict, Field, StrictStr

from algoliasearch.search.models.log_query import LogQuery


class Log(BaseModel):
    """
    Log
    """

    timestamp: StrictStr = Field(
        description="Date and time of the API request, in RFC 3339 format."
    )
    method: StrictStr = Field(description="HTTP method of the request.")
    answer_code: StrictStr = Field(description="HTTP status code of the response.")
    query_body: Annotated[str, Field(strict=True, max_length=1000)] = Field(
        description="Request body."
    )
    answer: Annotated[str, Field(strict=True, max_length=1000)] = Field(
        description="Response body."
    )
    url: StrictStr = Field(description="URL of the API endpoint.")
    ip: StrictStr = Field(
        description="IP address of the client that performed the request."
    )
    query_headers: StrictStr = Field(
        description="Request headers (API keys are obfuscated)."
    )
    sha1: StrictStr = Field(description="SHA1 signature of the log entry.")
    nb_api_calls: StrictStr = Field(description="Number of API requests.")
    processing_time_ms: StrictStr = Field(
        description="Processing time for the query in milliseconds. This doesn't include latency due to the network. "
    )
    index: Optional[StrictStr] = Field(
        default=None, description="Index targeted by the query."
    )
    var_query_params: Optional[StrictStr] = Field(
        default=None,
        description="Query parameters sent with the request.",
        alias="query_params",
    )
    query_nb_hits: Optional[StrictStr] = Field(
        default=None,
        description="Number of search results (hits) returned for the query.",
    )
    inner_queries: Optional[List[LogQuery]] = Field(
        default=None, description="Queries performed for the given request."
    )

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Log from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        _items = []
        if self.inner_queries:
            for _item in self.inner_queries:
                if _item:
                    _items.append(_item.to_dict())
            _dict["inner_queries"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of Log from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "timestamp": obj.get("timestamp"),
                "method": obj.get("method"),
                "answer_code": obj.get("answer_code"),
                "query_body": obj.get("query_body"),
                "answer": obj.get("answer"),
                "url": obj.get("url"),
                "ip": obj.get("ip"),
                "query_headers": obj.get("query_headers"),
                "sha1": obj.get("sha1"),
                "nb_api_calls": obj.get("nb_api_calls"),
                "processing_time_ms": obj.get("processing_time_ms"),
                "index": obj.get("index"),
                "query_params": obj.get("query_params"),
                "query_nb_hits": obj.get("query_nb_hits"),
                "inner_queries": (
                    [LogQuery.from_dict(_item) for _item in obj.get("inner_queries")]
                    if obj.get("inner_queries") is not None
                    else None
                ),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from re import match
from typing import Annotated, Any, Dict, Self

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator

from algoliasearch.search.models.user_highlight_result import UserHighlightResult


class UserHit(BaseModel):
    """
    UserHit
    """

    user_id: Annotated[str, Field(strict=True)] = Field(
        description="Unique identifier of the user who makes the search request.",
        alias="userID",
    )
    cluster_name: StrictStr = Field(description="Cluster name.", alias="clusterName")
    nb_records: StrictInt = Field(
        description="Number of records in the cluster.", alias="nbRecords"
    )
    data_size: StrictInt = Field(
        description="Data size taken by all the users assigned to the cluster.",
        alias="dataSize",
    )
    object_id: StrictStr = Field(
        description="userID of the requested user. Same as userID.", alias="objectID"
    )
    highlight_result: UserHighlightResult = Field(alias="_highlightResult")

    @field_validator("user_id")
    def user_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not match(r"^[a-zA-Z0-9 \-*.]+$", value):
            raise ValueError(
                r"must validate the regular expression /^[a-zA-Z0-9 \-*.]+$/"
            )
        return value

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of UserHit from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        if self.highlight_result:
            _dict["_highlightResult"] = self.highlight_result.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of UserHit from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "userID": obj.get("userID"),
                "clusterName": obj.get("clusterName"),
                "nbRecords": obj.get("nbRecords"),
                "dataSize": obj.get("dataSize"),
                "objectID": obj.get("objectID"),
                "_highlightResult": (
                    UserHighlightResult.from_dict(obj.get("_highlightResult"))
                    if obj.get("_highlightResult") is not None
                    else None
                ),
            }
        )
        return _obj



from os import environ
from typing import Optional

from algoliasearch.http.base_config import BaseConfig
from algoliasearch.http.hosts import CallType, Host, HostsCollection
from algoliasearch.http.user_agent import UserAgent


class RecommendConfig(BaseConfig):
    def __init__(
        self, app_id: Optional[str] = None, api_key: Optional[str] = None
    ) -> None:
        super().__init__(app_id, api_key)

        user_agent = UserAgent().add("Recommend")

        self.headers = {
            "x-algolia-application-id": app_id,
            "x-algolia-api-key": api_key,
            "user-agent": user_agent.get(),
            "content-type": "application/json",
        }

        self.proxies = {
            "http": environ.get("HTTP_PROXY"),
            "https": environ.get("HTTPS_PROXY"),
        }
        if self.proxies["http"] is None:
            del self.proxies["http"]
        if self.proxies["https"] is None:
            del self.proxies["https"]

        self.hosts = HostsCollection(
            [
                Host(
                    url="{}-dsn.algolia.net".format(self.app_id),
                    priority=10,
                    accept=CallType.READ,
                ),
                Host(
                    url="{}.algolia.net".format(self.app_id),
                    priority=10,
                    accept=CallType.WRITE,
                ),
                Host("{}-1.algolianet.com".format(self.app_id)),
                Host("{}-2.algolianet.com".format(self.app_id)),
                Host("{}-3.algolianet.com".format(self.app_id)),
            ],
            reorder_hosts=True,
        )



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import dumps
from typing import Annotated, Any, Dict, List, Optional, Self, Tuple, Union
from urllib.parse import quote

from pydantic import Field, StrictInt, StrictStr

from algoliasearch.http.api_response import ApiResponse
from algoliasearch.http.request_options import RequestOptions
from algoliasearch.http.serializer import bodySerializer
from algoliasearch.http.transporter import Transporter
from algoliasearch.http.verb import Verb
from algoliasearch.recommend.config import RecommendConfig
from algoliasearch.recommend.models.deleted_at_response import DeletedAtResponse
from algoliasearch.recommend.models.get_recommend_task_response import (
    GetRecommendTaskResponse,
)
from algoliasearch.recommend.models.get_recommendations_params import (
    GetRecommendationsParams,
)
from algoliasearch.recommend.models.get_recommendations_response import (
    GetRecommendationsResponse,
)
from algoliasearch.recommend.models.recommend_models import RecommendModels
from algoliasearch.recommend.models.recommend_rule import RecommendRule
from algoliasearch.recommend.models.search_recommend_rules_params import (
    SearchRecommendRulesParams,
)
from algoliasearch.recommend.models.search_recommend_rules_response import (
    SearchRecommendRulesResponse,
)


class RecommendClient:
    """The Algolia 'RecommendClient' class.

    Args:
    app_id (str): The Algolia application ID to retrieve information from.
    api_key (str): The Algolia api key bound to the given `app_id`.


    Returns:
    The initialized API client.

    Example:
    _client = RecommendClient("YOUR_ALGOLIA_APP_ID", "YOUR_ALGOLIA_API_KEY")
    _client_with_named_args = RecommendClient(app_id="YOUR_ALGOLIA_APP_ID", api_key="YOUR_ALGOLIA_API_KEY")

    See `RecommendClient.create_with_config` for advanced configuration.
    """

    _transporter: Transporter
    _config: RecommendConfig
    _request_options: RequestOptions

    def __init__(
        self,
        app_id: Optional[str] = None,
        api_key: Optional[str] = None,
        transporter: Optional[Transporter] = None,
        config: Optional[RecommendConfig] = None,
    ) -> None:
        if transporter is not None and config is None:
            config = transporter._config

        if config is None:
            config = RecommendConfig(app_id, api_key)
        self._config = config
        self._request_options = RequestOptions(config)

        if transporter is None:
            transporter = Transporter(config)
        self._transporter = transporter

    def create_with_config(
        config: RecommendConfig, transporter: Optional[Transporter] = None
    ) -> Self:
        """Allows creating a client with a customized `RecommendConfig` and `Transporter`. If `transporter` is not provided, the default one will be initialized from the given `config`.

        Args:
        config (RecommendConfig): The config of the API client.
        transporter (Transporter): The HTTP transporter, see `http/transporter.py` for implementation details.

        Returns:
        The initialized API client.

        Example:
        _client_with_custom_config = RecommendClient.create_with_config(config=RecommendConfig(...))
        _client_with_custom_config_and_transporter = RecommendClient.create_with_config(config=RecommendConfig(...), transporter=Transporter(...))
        """
        if transporter is None:
            transporter = Transporter(config)

        return RecommendClient(
            app_id=config.app_id,
            api_key=config.api_key,
            transporter=transporter,
            config=config,
        )

    async def __aenter__(self) -> None:
        return self

    async def __aexit__(self, exc_type, exc_value, traceback) -> None:
        """Closes the underlying `transporter` of the API client."""
        await self.close()

    async def close(self) -> None:
        """Closes the underlying `transporter` of the API client."""
        return await self._transporter.close()

    async def custom_delete_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError(
                "Parameter `path` is required when calling `custom_delete`."
            )

        _query_parameters: List[Tuple[str, str]] = []

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters.append((_qpkey, _qpvalue))

        return await self._transporter.request(
            verb=Verb.DELETE,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_delete(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        return (
            await self.custom_delete_with_http_info(path, parameters, request_options)
        ).deserialize(object)

    async def custom_get_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_get`.")

        _query_parameters: List[Tuple[str, str]] = []

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters.append((_qpkey, _qpvalue))

        return await self._transporter.request(
            verb=Verb.GET,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_get(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        return (
            await self.custom_get_with_http_info(path, parameters, request_options)
        ).deserialize(object)

    async def custom_post_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_post`.")

        _query_parameters: List[Tuple[str, str]] = []

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters.append((_qpkey, _qpvalue))

        _data = {}
        if body is not None:
            _data = body

        return await self._transporter.request(
            verb=Verb.POST,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                data=dumps(bodySerializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_post(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        return (
            await self.custom_post_with_http_info(
                path, parameters, body, request_options
            )
        ).deserialize(object)

    async def custom_put_with_http_info(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if path is None:
            raise ValueError("Parameter `path` is required when calling `custom_put`.")

        _query_parameters: List[Tuple[str, str]] = []

        if parameters is not None:
            for _qpkey, _qpvalue in parameters.items():
                _query_parameters.append((_qpkey, _qpvalue))

        _data = {}
        if body is not None:
            _data = body

        return await self._transporter.request(
            verb=Verb.PUT,
            path="/{path}".replace("{path}", path),
            request_options=self._request_options.merge(
                query_parameters=_query_parameters,
                data=dumps(bodySerializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def custom_put(
        self,
        path: Annotated[
            StrictStr,
            Field(
                description='Path of the endpoint, anything after "/1" must be specified.'
            ),
        ],
        parameters: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Query parameters to apply to the current query."),
        ] = None,
        body: Annotated[
            Optional[Dict[str, Any]],
            Field(description="Parameters to send with the custom request."),
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> object:
        """
        This method allow you to send requests to the Algolia REST API.


        :param path: Path of the endpoint, anything after \"/1\" must be specified. (required)
        :type path: str
        :param parameters: Query parameters to apply to the current query.
        :type parameters: Dict[str, object]
        :param body: Parameters to send with the custom request.
        :type body: object
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'object' result object.
        """
        return (
            await self.custom_put_with_http_info(
                path, parameters, body, request_options
            )
        ).deserialize(object)

    async def delete_recommend_rule_with_http_info(
        self,
        index_name: Annotated[
            StrictStr,
            Field(description="Name of the index on which to perform the operation."),
        ],
        model: Annotated[
            RecommendModels,
            Field(
                description="[Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models). "
            ),
        ],
        object_id: Annotated[StrictStr, Field(description="Unique record identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Deletes a Recommend rule from a recommendation scenario.

        Required API Key ACLs:
          - editSettings

        :param index_name: Name of the index on which to perform the operation. (required)
        :type index_name: str
        :param model: [Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models).  (required)
        :type model: RecommendModels
        :param object_id: Unique record identifier. (required)
        :type object_id: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if index_name is None:
            raise ValueError(
                "Parameter `index_name` is required when calling `delete_recommend_rule`."
            )

        if model is None:
            raise ValueError(
                "Parameter `model` is required when calling `delete_recommend_rule`."
            )

        if object_id is None:
            raise ValueError(
                "Parameter `object_id` is required when calling `delete_recommend_rule`."
            )

        return await self._transporter.request(
            verb=Verb.DELETE,
            path="/1/indexes/{indexName}/{model}/recommend/rules/{objectID}".replace(
                "{indexName}", quote(str(index_name), safe="")
            )
            .replace("{model}", quote(str(model), safe=""))
            .replace("{objectID}", quote(str(object_id), safe="")),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def delete_recommend_rule(
        self,
        index_name: Annotated[
            StrictStr,
            Field(description="Name of the index on which to perform the operation."),
        ],
        model: Annotated[
            RecommendModels,
            Field(
                description="[Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models). "
            ),
        ],
        object_id: Annotated[StrictStr, Field(description="Unique record identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> DeletedAtResponse:
        """
        Deletes a Recommend rule from a recommendation scenario.

        Required API Key ACLs:
          - editSettings

        :param index_name: Name of the index on which to perform the operation. (required)
        :type index_name: str
        :param model: [Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models).  (required)
        :type model: RecommendModels
        :param object_id: Unique record identifier. (required)
        :type object_id: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'DeletedAtResponse' result object.
        """
        return (
            await self.delete_recommend_rule_with_http_info(
                index_name, model, object_id, request_options
            )
        ).deserialize(DeletedAtResponse)

    async def get_recommend_rule_with_http_info(
        self,
        index_name: Annotated[
            StrictStr,
            Field(description="Name of the index on which to perform the operation."),
        ],
        model: Annotated[
            RecommendModels,
            Field(
                description="[Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models). "
            ),
        ],
        object_id: Annotated[StrictStr, Field(description="Unique record identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Retrieves a Recommend rule that you previously created in the Algolia dashboard.

        Required API Key ACLs:
          - settings

        :param index_name: Name of the index on which to perform the operation. (required)
        :type index_name: str
        :param model: [Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models).  (required)
        :type model: RecommendModels
        :param object_id: Unique record identifier. (required)
        :type object_id: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if index_name is None:
            raise ValueError(
                "Parameter `index_name` is required when calling `get_recommend_rule`."
            )

        if model is None:
            raise ValueError(
                "Parameter `model` is required when calling `get_recommend_rule`."
            )

        if object_id is None:
            raise ValueError(
                "Parameter `object_id` is required when calling `get_recommend_rule`."
            )

        return await self._transporter.request(
            verb=Verb.GET,
            path="/1/indexes/{indexName}/{model}/recommend/rules/{objectID}".replace(
                "{indexName}", quote(str(index_name), safe="")
            )
            .replace("{model}", quote(str(model), safe=""))
            .replace("{objectID}", quote(str(object_id), safe="")),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def get_recommend_rule(
        self,
        index_name: Annotated[
            StrictStr,
            Field(description="Name of the index on which to perform the operation."),
        ],
        model: Annotated[
            RecommendModels,
            Field(
                description="[Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models). "
            ),
        ],
        object_id: Annotated[StrictStr, Field(description="Unique record identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> RecommendRule:
        """
        Retrieves a Recommend rule that you previously created in the Algolia dashboard.

        Required API Key ACLs:
          - settings

        :param index_name: Name of the index on which to perform the operation. (required)
        :type index_name: str
        :param model: [Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models).  (required)
        :type model: RecommendModels
        :param object_id: Unique record identifier. (required)
        :type object_id: str
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'RecommendRule' result object.
        """
        return (
            await self.get_recommend_rule_with_http_info(
                index_name, model, object_id, request_options
            )
        ).deserialize(RecommendRule)

    async def get_recommend_status_with_http_info(
        self,
        index_name: Annotated[
            StrictStr,
            Field(description="Name of the index on which to perform the operation."),
        ],
        model: Annotated[
            RecommendModels,
            Field(
                description="[Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models). "
            ),
        ],
        task_id: Annotated[StrictInt, Field(description="Unique task identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Checks the status of a given task.  Deleting a Recommend rule is asynchronous. When you delete a rule, a task is created on a queue and completed depending on the load on the server. The API response includes a task ID that you can use to check the status.

        Required API Key ACLs:
          - editSettings

        :param index_name: Name of the index on which to perform the operation. (required)
        :type index_name: str
        :param model: [Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models).  (required)
        :type model: RecommendModels
        :param task_id: Unique task identifier. (required)
        :type task_id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if index_name is None:
            raise ValueError(
                "Parameter `index_name` is required when calling `get_recommend_status`."
            )

        if model is None:
            raise ValueError(
                "Parameter `model` is required when calling `get_recommend_status`."
            )

        if task_id is None:
            raise ValueError(
                "Parameter `task_id` is required when calling `get_recommend_status`."
            )

        return await self._transporter.request(
            verb=Verb.GET,
            path="/1/indexes/{indexName}/{model}/task/{taskID}".replace(
                "{indexName}", quote(str(index_name), safe="")
            )
            .replace("{model}", quote(str(model), safe=""))
            .replace("{taskID}", quote(str(task_id), safe="")),
            request_options=self._request_options.merge(
                user_request_options=request_options,
            ),
            use_read_transporter=False,
        )

    async def get_recommend_status(
        self,
        index_name: Annotated[
            StrictStr,
            Field(description="Name of the index on which to perform the operation."),
        ],
        model: Annotated[
            RecommendModels,
            Field(
                description="[Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models). "
            ),
        ],
        task_id: Annotated[StrictInt, Field(description="Unique task identifier.")],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> GetRecommendTaskResponse:
        """
        Checks the status of a given task.  Deleting a Recommend rule is asynchronous. When you delete a rule, a task is created on a queue and completed depending on the load on the server. The API response includes a task ID that you can use to check the status.

        Required API Key ACLs:
          - editSettings

        :param index_name: Name of the index on which to perform the operation. (required)
        :type index_name: str
        :param model: [Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models).  (required)
        :type model: RecommendModels
        :param task_id: Unique task identifier. (required)
        :type task_id: int
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'GetRecommendTaskResponse' result object.
        """
        return (
            await self.get_recommend_status_with_http_info(
                index_name, model, task_id, request_options
            )
        ).deserialize(GetRecommendTaskResponse)

    async def get_recommendations_with_http_info(
        self,
        get_recommendations_params: GetRecommendationsParams,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Retrieves recommendations from selected AI models.

        Required API Key ACLs:
          - search

        :param get_recommendations_params: (required)
        :type get_recommendations_params: GetRecommendationsParams
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if get_recommendations_params is None:
            raise ValueError(
                "Parameter `get_recommendations_params` is required when calling `get_recommendations`."
            )

        _data = {}
        if get_recommendations_params is not None:
            _data = get_recommendations_params

        return await self._transporter.request(
            verb=Verb.POST,
            path="/1/indexes/*/recommendations",
            request_options=self._request_options.merge(
                data=dumps(bodySerializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=True,
        )

    async def get_recommendations(
        self,
        get_recommendations_params: GetRecommendationsParams,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> GetRecommendationsResponse:
        """
        Retrieves recommendations from selected AI models.

        Required API Key ACLs:
          - search

        :param get_recommendations_params: (required)
        :type get_recommendations_params: GetRecommendationsParams
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'GetRecommendationsResponse' result object.
        """
        return (
            await self.get_recommendations_with_http_info(
                get_recommendations_params, request_options
            )
        ).deserialize(GetRecommendationsResponse)

    async def search_recommend_rules_with_http_info(
        self,
        index_name: Annotated[
            StrictStr,
            Field(description="Name of the index on which to perform the operation."),
        ],
        model: Annotated[
            RecommendModels,
            Field(
                description="[Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models). "
            ),
        ],
        search_recommend_rules_params: Optional[SearchRecommendRulesParams] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Searches for Recommend rules.  Use an empty query to list all rules for this recommendation scenario.

        Required API Key ACLs:
          - settings

        :param index_name: Name of the index on which to perform the operation. (required)
        :type index_name: str
        :param model: [Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models).  (required)
        :type model: RecommendModels
        :param search_recommend_rules_params:
        :type search_recommend_rules_params: SearchRecommendRulesParams
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if index_name is None:
            raise ValueError(
                "Parameter `index_name` is required when calling `search_recommend_rules`."
            )

        if model is None:
            raise ValueError(
                "Parameter `model` is required when calling `search_recommend_rules`."
            )

        _data = {}
        if search_recommend_rules_params is not None:
            _data = search_recommend_rules_params

        return await self._transporter.request(
            verb=Verb.POST,
            path="/1/indexes/{indexName}/{model}/recommend/rules/search".replace(
                "{indexName}", quote(str(index_name), safe="")
            ).replace("{model}", quote(str(model), safe="")),
            request_options=self._request_options.merge(
                data=dumps(bodySerializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=True,
        )

    async def search_recommend_rules(
        self,
        index_name: Annotated[
            StrictStr,
            Field(description="Name of the index on which to perform the operation."),
        ],
        model: Annotated[
            RecommendModels,
            Field(
                description="[Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models). "
            ),
        ],
        search_recommend_rules_params: Optional[SearchRecommendRulesParams] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> SearchRecommendRulesResponse:
        """
        Searches for Recommend rules.  Use an empty query to list all rules for this recommendation scenario.

        Required API Key ACLs:
          - settings

        :param index_name: Name of the index on which to perform the operation. (required)
        :type index_name: str
        :param model: [Recommend model](https://www.algolia.com/doc/guides/algolia-recommend/overview/#recommend-models).  (required)
        :type model: RecommendModels
        :param search_recommend_rules_params:
        :type search_recommend_rules_params: SearchRecommendRulesParams
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'SearchRecommendRulesResponse' result object.
        """
        return (
            await self.search_recommend_rules_with_http_info(
                index_name, model, search_recommend_rules_params, request_options
            )
        ).deserialize(SearchRecommendRulesResponse)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from enum import Enum
from json import loads
from typing import Self


class ClickEvent(str, Enum):
    """
    ClickEvent
    """

    """
    allowed enum values
    """
    CLICK = "click"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ClickEvent from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from enum import Enum
from json import loads
from typing import Self


class SortRemainingBy(str, Enum):
    """
    Order of facet values that aren't explicitly positioned with the `order` setting.  - `count`.   Order remaining facet values by decreasing count.   The count is the number of matching records containing this facet value.  - `alpha`.   Sort facet values alphabetically.  - `hidden`.   Don't show facet values that aren't explicitly positioned.
    """

    """
    allowed enum values
    """
    COUNT = "count"
    ALPHA = "alpha"
    HIDDEN = "hidden"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SortRemainingBy from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from typing import Annotated, Any, Dict, List, Self

from pydantic import BaseModel, ConfigDict, Field

from algoliasearch.personalization.models.event_scoring import EventScoring
from algoliasearch.personalization.models.facet_scoring import FacetScoring


class PersonalizationStrategyParams(BaseModel):
    """
    PersonalizationStrategyParams
    """

    event_scoring: List[EventScoring] = Field(
        description="Scores associated with each event.  The higher the scores, the higher the impact of those events on the personalization of search results. ",
        alias="eventScoring",
    )
    facet_scoring: List[FacetScoring] = Field(
        description="Scores associated with each facet.  The higher the scores, the higher the impact of those events on the personalization of search results. ",
        alias="facetScoring",
    )
    personalization_impact: Annotated[int, Field(le=100, strict=True, ge=0)] = Field(
        description="Impact of personalization on the search results.  If set to 0, personalization has no impact on the search results. ",
        alias="personalizationImpact",
    )

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PersonalizationStrategyParams from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        _items = []
        if self.event_scoring:
            for _item in self.event_scoring:
                if _item:
                    _items.append(_item.to_dict())
            _dict["eventScoring"] = _items
        _items = []
        if self.facet_scoring:
            for _item in self.facet_scoring:
                if _item:
                    _items.append(_item.to_dict())
            _dict["facetScoring"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of PersonalizationStrategyParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "eventScoring": (
                    [EventScoring.from_dict(_item) for _item in obj.get("eventScoring")]
                    if obj.get("eventScoring") is not None
                    else None
                ),
                "facetScoring": (
                    [FacetScoring.from_dict(_item) for _item in obj.get("facetScoring")]
                    if obj.get("facetScoring") is not None
                    else None
                ),
                "personalizationImpact": obj.get("personalizationImpact"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from re import match
from typing import Annotated, Any, Dict, Optional, Self

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator


class Condition(BaseModel):
    """
    Condition that triggers the rule. If not specified, the rule is triggered for all recommendations.
    """

    filters: Optional[StrictStr] = Field(
        default=None,
        description="Filter expression to only include items that match the filter criteria in the response.  You can use these filter expressions:  - **Numeric filters.** `<facet> <op> <number>`, where `<op>` is one of `<`, `<=`, `=`, `!=`, `>`, `>=`. - **Ranges.** `<facet>:<lower> TO <upper>` where `<lower>` and `<upper>` are the lower and upper limits of the range (inclusive). - **Facet filters.** `<facet>:<value>` where `<facet>` is a facet attribute (case-sensitive) and `<value>` a facet value. - **Tag filters.** `_tags:<value>` or just `<value>` (case-sensitive). - **Boolean filters.** `<facet>: true | false`.  You can combine filters with `AND`, `OR`, and `NOT` operators with the following restrictions:  - You can only combine filters of the same type with `OR`.   **Not supported:** `facet:value OR num > 3`. - You can't use `NOT` with combinations of filters.   **Not supported:** `NOT(facet:value OR facet:value)` - You can't combine conjunctions (`AND`) with `OR`.   **Not supported:** `facet:value OR (facet:value AND facet:value)`  Use quotes around your filters, if the facet attribute name or facet value has spaces, keywords (`OR`, `AND`, `NOT`), or quotes. If a facet attribute is an array, the filter matches if it matches at least one element of the array.  For more information, see [Filters](https://www.algolia.com/doc/guides/managing-results/refine-results/filtering/). ",
    )
    context: Optional[Annotated[str, Field(strict=True)]] = Field(
        default=None,
        description="An additional restriction that only triggers the rule, when the search has the same value as `ruleContexts` parameter. For example, if `context: mobile`, the rule is only triggered when the search request has a matching `ruleContexts: mobile`. A rule context must only contain alphanumeric characters. ",
    )

    @field_validator("context")
    def context_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not match(r"[A-Za-z0-9_-]+", value):
            raise ValueError(r"must validate the regular expression /[A-Za-z0-9_-]+/")
        return value

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Condition from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of Condition from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {"filters": obj.get("filters"), "context": obj.get("context")}
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from typing import Annotated, Any, Dict, List, Optional, Self

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr

from algoliasearch.search.models.supported_language import SupportedLanguage


class BaseIndexSettings(BaseModel):
    """
    BaseIndexSettings
    """

    attributes_for_faceting: Optional[List[StrictStr]] = Field(
        default=None,
        description='Attributes used for [faceting](https://www.algolia.com/doc/guides/managing-results/refine-results/faceting/).  Facets are attributes that let you categorize search results. They can be used for filtering search results. By default, no attribute is used for faceting. Attribute names are case-sensitive.  **Modifiers**  - `filterOnly("ATTRIBUTE")`.   Allows using this attribute as a filter, but doesn\'t evalue the facet values.  - `searchable("ATTRIBUTE")`.   Allows searching for facet values.  - `afterDistinct("ATTRIBUTE")`.   Evaluates the facet count _after_ deduplication with `distinct`.   This ensures accurate facet counts.   You can apply this modifier to searchable facets: `afterDistinct(searchable(ATTRIBUTE))`. ',
        alias="attributesForFaceting",
    )
    replicas: Optional[List[StrictStr]] = Field(
        default=None,
        description="Creates [replica indices](https://www.algolia.com/doc/guides/managing-results/refine-results/sorting/in-depth/replicas/).  Replicas are copies of a primary index with the same records but different settings, synonyms, or rules. If you want to offer a different ranking or sorting of your search results, you'll use replica indices. All index operations on a primary index are automatically forwarded to its replicas. To add a replica index, you must provide the complete set of replicas to this parameter. If you omit a replica from this list, the replica turns into a regular, standalone index that will no longer by synced with the primary index.  **Modifier**  - `virtual(\"REPLICA\")`.   Create a virtual replica,   Virtual replicas don't increase the number of records and are optimized for [Relevant sorting](https://www.algolia.com/doc/guides/managing-results/refine-results/sorting/in-depth/relevant-sort/). ",
    )
    pagination_limited_to: Optional[Annotated[int, Field(le=20000, strict=True)]] = (
        Field(
            default=1000,
            description="Maximum number of search results that can be obtained through pagination.  Higher pagination limits might slow down your search. For pagination limits above 1,000, the sorting of results beyond the 1,000th hit can't be guaranteed. ",
            alias="paginationLimitedTo",
        )
    )
    unretrievable_attributes: Optional[List[StrictStr]] = Field(
        default=None,
        description="Attributes that can't be retrieved at query time.  This can be useful if you want to use an attribute for ranking or to [restrict access](https://www.algolia.com/doc/guides/security/api-keys/how-to/user-restricted-access-to-data/), but don't want to include it in the search results. Attribute names are case-sensitive. ",
        alias="unretrievableAttributes",
    )
    disable_typo_tolerance_on_words: Optional[List[StrictStr]] = Field(
        default=None,
        description="Words for which you want to turn off [typo tolerance](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/typo-tolerance/). This also turns off [word splitting and concatenation](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/splitting-and-concatenation/) for the specified words. ",
        alias="disableTypoToleranceOnWords",
    )
    attributes_to_transliterate: Optional[List[StrictStr]] = Field(
        default=None,
        description="Attributes, for which you want to support [Japanese transliteration](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/language-specific-configurations/#japanese-transliteration-and-type-ahead).  Transliteration supports searching in any of the Japanese writing systems. To support transliteration, you must set the indexing language to Japanese. Attribute names are case-sensitive. ",
        alias="attributesToTransliterate",
    )
    camel_case_attributes: Optional[List[StrictStr]] = Field(
        default=None,
        description="Attributes for which to split [camel case](https://wikipedia.org/wiki/Camel_case) words. Attribute names are case-sensitive. ",
        alias="camelCaseAttributes",
    )
    decompounded_attributes: Optional[Dict[str, Any]] = Field(
        default=None,
        description='Searchable attributes to which Algolia should apply [word segmentation](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/how-to/customize-segmentation/) (decompounding). Attribute names are case-sensitive.  Compound words are formed by combining two or more individual words, and are particularly prevalent in Germanic languages—for example, "firefighter". With decompounding, the individual components are indexed separately.  You can specify different lists for different languages. Decompounding is supported for these languages: Dutch (`nl`), German (`de`), Finnish (`fi`), Danish (`da`), Swedish (`sv`), and Norwegian (`no`). ',
        alias="decompoundedAttributes",
    )
    index_languages: Optional[List[SupportedLanguage]] = Field(
        default=None,
        description="Languages for language-specific processing steps, such as word detection and dictionary settings.  **You should always specify an indexing language.** If you don't specify an indexing language, the search engine uses all [supported languages](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/supported-languages/), or the languages you specified with the `ignorePlurals` or `removeStopWords` parameters. This can lead to unexpected search results. For more information, see [Language-specific configuration](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/language-specific-configurations/). ",
        alias="indexLanguages",
    )
    disable_prefix_on_attributes: Optional[List[StrictStr]] = Field(
        default=None,
        description="Searchable attributes for which you want to turn off [prefix matching](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/override-search-engine-defaults/#adjusting-prefix-search). Attribute names are case-sensitive. ",
        alias="disablePrefixOnAttributes",
    )
    allow_compression_of_integer_array: Optional[StrictBool] = Field(
        default=False,
        description="Whether arrays with exclusively non-negative integers should be compressed for better performance. If true, the compressed arrays may be reordered. ",
        alias="allowCompressionOfIntegerArray",
    )
    numeric_attributes_for_filtering: Optional[List[StrictStr]] = Field(
        default=None,
        description='Numeric attributes that can be used as [numerical filters](https://www.algolia.com/doc/guides/managing-results/rules/detecting-intent/how-to/applying-a-custom-filter-for-a-specific-query/#numerical-filters). Attribute names are case-sensitive.  By default, all numeric attributes are available as numerical filters. For faster indexing, reduce the number of numeric attributes.  If you want to turn off filtering for all numeric attributes, specifiy an attribute that doesn\'t exist in your index, such as `NO_NUMERIC_FILTERING`.  **Modifier**  - `equalOnly("ATTRIBUTE")`.   Support only filtering based on equality comparisons `=` and `!=`. ',
        alias="numericAttributesForFiltering",
    )
    separators_to_index: Optional[StrictStr] = Field(
        default="",
        description="Controls which separators are indexed.  Separators are all non-letter characters except spaces and currency characters, such as $€£¥. By default, separator characters aren't indexed. With `separatorsToIndex`, Algolia treats separator characters as separate words. For example, a search for `C#` would report two matches. ",
        alias="separatorsToIndex",
    )
    searchable_attributes: Optional[List[StrictStr]] = Field(
        default=None,
        description='Attributes used for searching. Attribute names are case-sensitive.  By default, all attributes are searchable and the [Attribute](https://www.algolia.com/doc/guides/managing-results/relevance-overview/in-depth/ranking-criteria/#attribute) ranking criterion is turned off. With a non-empty list, Algolia only returns results with matches in the selected attributes. In addition, the Attribute ranking criterion is turned on: matches in attributes that are higher in the list of `searchableAttributes` rank first. To make matches in two attributes rank equally, include them in a comma-separated string, such as `"title,alternate_title"`. Attributes with the same priority are always unordered.  For more information, see [Searchable attributes](https://www.algolia.com/doc/guides/sending-and-managing-data/prepare-your-data/how-to/setting-searchable-attributes/).  **Modifier**  - `unordered("ATTRIBUTE")`.   Ignore the position of a match within the attribute.  Without modifier, matches at the beginning of an attribute rank higer than matches at the end. ',
        alias="searchableAttributes",
    )
    user_data: Optional[Dict[str, Any]] = Field(
        default=None,
        description="An object with custom data.  You can store up to 32kB as custom data. ",
        alias="userData",
    )
    custom_normalization: Optional[Dict[str, Dict[str, StrictStr]]] = Field(
        default=None,
        description="Characters and their normalized replacements. This overrides Algolia's default [normalization](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/in-depth/normalization/). ",
        alias="customNormalization",
    )
    attribute_for_distinct: Optional[StrictStr] = Field(
        default=None,
        description="Attribute that should be used to establish groups of results. Attribute names are case-sensitive.  All records with the same value for this attribute are considered a group. You can combine `attributeForDistinct` with the `distinct` search parameter to control how many items per group are included in the search results.  If you want to use the same attribute also for faceting, use the `afterDistinct` modifier of the `attributesForFaceting` setting. This applies faceting _after_ deduplication, which will result in accurate facet counts. ",
        alias="attributeForDistinct",
    )

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of BaseIndexSettings from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of BaseIndexSettings from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "attributesForFaceting": obj.get("attributesForFaceting"),
                "replicas": obj.get("replicas"),
                "paginationLimitedTo": obj.get("paginationLimitedTo"),
                "unretrievableAttributes": obj.get("unretrievableAttributes"),
                "disableTypoToleranceOnWords": obj.get("disableTypoToleranceOnWords"),
                "attributesToTransliterate": obj.get("attributesToTransliterate"),
                "camelCaseAttributes": obj.get("camelCaseAttributes"),
                "decompoundedAttributes": obj.get("decompoundedAttributes"),
                "indexLanguages": obj.get("indexLanguages"),
                "disablePrefixOnAttributes": obj.get("disablePrefixOnAttributes"),
                "allowCompressionOfIntegerArray": obj.get(
                    "allowCompressionOfIntegerArray"
                ),
                "numericAttributesForFiltering": obj.get(
                    "numericAttributesForFiltering"
                ),
                "separatorsToIndex": obj.get("separatorsToIndex"),
                "searchableAttributes": obj.get("searchableAttributes"),
                "userData": obj.get("userData"),
                "customNormalization": obj.get("customNormalization"),
                "attributeForDistinct": obj.get("attributeForDistinct"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from typing import Annotated, Any, Dict, List, Optional, Self

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr

from algoliasearch.ingestion.models.commercetools_custom_fields import (
    CommercetoolsCustomFields,
)


class SourceUpdateCommercetools(BaseModel):
    """
    SourceUpdateCommercetools
    """

    store_keys: Optional[List[StrictStr]] = Field(default=None, alias="storeKeys")
    locales: Optional[List[Annotated[str, Field(strict=True)]]] = Field(
        default=None, description="Locales for your commercetools stores."
    )
    url: Optional[StrictStr] = None
    fallback_is_in_stock_value: Optional[StrictBool] = Field(
        default=None,
        description="Whether a fallback value is stored in the Algolia record if there's no inventory information about the product. ",
        alias="fallbackIsInStockValue",
    )
    custom_fields: Optional[CommercetoolsCustomFields] = Field(
        default=None, alias="customFields"
    )

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SourceUpdateCommercetools from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        if self.custom_fields:
            _dict["customFields"] = self.custom_fields.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of SourceUpdateCommercetools from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "storeKeys": obj.get("storeKeys"),
                "locales": obj.get("locales"),
                "url": obj.get("url"),
                "fallbackIsInStockValue": obj.get("fallbackIsInStockValue"),
                "customFields": (
                    CommercetoolsCustomFields.from_dict(obj.get("customFields"))
                    if obj.get("customFields") is not None
                    else None
                ),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from typing import Annotated, Any, Dict, Optional, Self, Union

from pydantic import BaseModel, ConfigDict, Field, StrictStr

from algoliasearch.recommend.models.fallback_params import FallbackParams
from algoliasearch.recommend.models.recommended_for_you_model import (
    RecommendedForYouModel,
)
from algoliasearch.recommend.models.search_params import SearchParams


class RecommendedForYouQuery(BaseModel):
    """
    RecommendedForYouQuery
    """

    index_name: StrictStr = Field(
        description="Index name (case-sensitive).", alias="indexName"
    )
    threshold: Union[
        Annotated[float, Field(le=100, strict=True, ge=0)],
        Annotated[int, Field(le=100, strict=True, ge=0)],
    ] = Field(
        description="Minimum score a recommendation must have to be included in the response."
    )
    max_recommendations: Optional[Annotated[int, Field(le=1000, strict=True, ge=1)]] = (
        Field(
            default=30,
            description="Maximum number of recommendations to retrieve. By default, all recommendations are returned and no fallback request is made. Depending on the available recommendations and the other request parameters, the actual number of recommendations may be lower than this value. ",
            alias="maxRecommendations",
        )
    )
    query_parameters: Optional[SearchParams] = Field(
        default=None, alias="queryParameters"
    )
    model: RecommendedForYouModel
    fallback_parameters: Optional[FallbackParams] = Field(
        default=None, alias="fallbackParameters"
    )

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of RecommendedForYouQuery from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        if self.query_parameters:
            _dict["queryParameters"] = self.query_parameters.to_dict()
        if self.fallback_parameters:
            _dict["fallbackParameters"] = self.fallback_parameters.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of RecommendedForYouQuery from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "indexName": obj.get("indexName"),
                "threshold": obj.get("threshold"),
                "maxRecommendations": obj.get("maxRecommendations"),
                "queryParameters": (
                    SearchParams.from_dict(obj.get("queryParameters"))
                    if obj.get("queryParameters") is not None
                    else None
                ),
                "model": obj.get("model"),
                "fallbackParameters": (
                    FallbackParams.from_dict(obj.get("fallbackParameters"))
                    if obj.get("fallbackParameters") is not None
                    else None
                ),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from typing import Annotated, Any, Dict, Optional, Self, Union

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr

from algoliasearch.abtesting.models.currency import Currency
from algoliasearch.abtesting.models.filter_effects import FilterEffects


class Variant(BaseModel):
    """
    Variant
    """

    add_to_cart_count: StrictInt = Field(
        description="Number of add-to-cart events for this variant.",
        alias="addToCartCount",
    )
    add_to_cart_rate: Optional[Union[StrictFloat, StrictInt]] = Field(
        description="[Add-to-cart rate](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#add-to-cart-rate) for this variant. ",
        alias="addToCartRate",
    )
    average_click_position: Optional[StrictInt] = Field(
        description="[Average click position](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#click-position) for this variant. ",
        alias="averageClickPosition",
    )
    click_count: StrictInt = Field(
        description="Number of click events for this variant.", alias="clickCount"
    )
    click_through_rate: Optional[Union[StrictFloat, StrictInt]] = Field(
        description="[Click-through rate](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#click-through-rate) for this variant. ",
        alias="clickThroughRate",
    )
    conversion_count: StrictInt = Field(
        description="Number of click events for this variant.", alias="conversionCount"
    )
    conversion_rate: Optional[Union[StrictFloat, StrictInt]] = Field(
        description="[Conversion rate](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#conversion-rate) for this variant. ",
        alias="conversionRate",
    )
    currencies: Optional[Dict[str, Currency]] = Field(
        default=None, description="A/B test currencies."
    )
    description: StrictStr = Field(description="Description for this variant.")
    estimated_sample_size: Optional[StrictInt] = Field(
        default=None,
        description="Estimated number of searches required to achieve the desired statistical significance.  The A/B test configuration must include a `mininmumDetectableEffect` setting for this number to be included in the response. ",
        alias="estimatedSampleSize",
    )
    filter_effects: Optional[FilterEffects] = Field(default=None, alias="filterEffects")
    index: StrictStr = Field(
        description="Index name of the A/B test variant (case-sensitive)."
    )
    no_result_count: Optional[StrictInt] = Field(
        description="Number of [searches without results](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#searches-without-results) for this variant.",
        alias="noResultCount",
    )
    purchase_count: StrictInt = Field(
        description="Number of purchase events for this variant.", alias="purchaseCount"
    )
    purchase_rate: Optional[Union[StrictFloat, StrictInt]] = Field(
        description="[Purchase rate](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#purchase-rate) for this variant. ",
        alias="purchaseRate",
    )
    search_count: Optional[StrictInt] = Field(
        description="Number of searches for this variant.", alias="searchCount"
    )
    tracked_search_count: Optional[StrictInt] = Field(
        default=0,
        description="Number of tracked searches. Tracked searches are search requests where the `clickAnalytics` parameter is true.",
        alias="trackedSearchCount",
    )
    traffic_percentage: Annotated[int, Field(le=100, strict=True, ge=0)] = Field(
        description="Percentage of search requests each variant receives.",
        alias="trafficPercentage",
    )
    user_count: Optional[StrictInt] = Field(
        description="Number of users that made searches to this variant.",
        alias="userCount",
    )
    tracked_user_count: Optional[StrictInt] = Field(
        description="Number of users that made tracked searches to this variant.",
        alias="trackedUserCount",
    )

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Variant from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        _field_dict = {}
        if self.currencies:
            for _key in self.currencies:
                if self.currencies[_key]:
                    _field_dict[_key] = self.currencies[_key].to_dict()
            _dict["currencies"] = _field_dict
        if self.filter_effects:
            _dict["filterEffects"] = self.filter_effects.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of Variant from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "addToCartCount": obj.get("addToCartCount"),
                "addToCartRate": obj.get("addToCartRate"),
                "averageClickPosition": obj.get("averageClickPosition"),
                "clickCount": obj.get("clickCount"),
                "clickThroughRate": obj.get("clickThroughRate"),
                "conversionCount": obj.get("conversionCount"),
                "conversionRate": obj.get("conversionRate"),
                "currencies": (
                    dict(
                        (_k, Currency.from_dict(_v))
                        for _k, _v in obj.get("currencies").items()
                    )
                    if obj.get("currencies") is not None
                    else None
                ),
                "description": obj.get("description"),
                "estimatedSampleSize": obj.get("estimatedSampleSize"),
                "filterEffects": (
                    FilterEffects.from_dict(obj.get("filterEffects"))
                    if obj.get("filterEffects") is not None
                    else None
                ),
                "index": obj.get("index"),
                "noResultCount": obj.get("noResultCount"),
                "purchaseCount": obj.get("purchaseCount"),
                "purchaseRate": obj.get("purchaseRate"),
                "searchCount": obj.get("searchCount"),
                "trackedSearchCount": obj.get("trackedSearchCount"),
                "trafficPercentage": obj.get("trafficPercentage"),
                "userCount": obj.get("userCount"),
                "trackedUserCount": obj.get("trackedUserCount"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from typing import Any, Dict, List, Optional, Self

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr


class FetchedIndex(BaseModel):
    """
    FetchedIndex
    """

    name: StrictStr = Field(description="Index name.")
    created_at: StrictStr = Field(
        description="Index creation date. An empty string means that the index has no records.",
        alias="createdAt",
    )
    updated_at: StrictStr = Field(
        description="Date and time when the object was updated, in RFC 3339 format.",
        alias="updatedAt",
    )
    entries: StrictInt = Field(description="Number of records contained in the index.")
    data_size: StrictInt = Field(
        description="Number of bytes of the index in minified format.", alias="dataSize"
    )
    file_size: StrictInt = Field(
        description="Number of bytes of the index binary file.", alias="fileSize"
    )
    last_build_time_s: StrictInt = Field(
        description="Last build time.", alias="lastBuildTimeS"
    )
    number_of_pending_tasks: StrictInt = Field(
        description="Number of pending indexing operations. This value is deprecated and should not be used.",
        alias="numberOfPendingTasks",
    )
    pending_task: StrictBool = Field(
        description="A boolean which says whether the index has pending tasks. This value is deprecated and should not be used.",
        alias="pendingTask",
    )
    primary: Optional[StrictStr] = Field(
        default=None,
        description="Only present if the index is a replica. Contains the name of the related primary index.",
    )
    replicas: Optional[List[StrictStr]] = Field(
        default=None,
        description="Only present if the index is a primary index with replicas. Contains the names of all linked replicas.",
    )
    virtual: Optional[StrictBool] = Field(
        default=None,
        description="Only present if the index is a [virtual replica](https://www.algolia.com/doc/guides/managing-results/refine-results/sorting/how-to/sort-an-index-alphabetically/#virtual-replicas).",
    )

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of FetchedIndex from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of FetchedIndex from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "name": obj.get("name"),
                "createdAt": obj.get("createdAt"),
                "updatedAt": obj.get("updatedAt"),
                "entries": obj.get("entries"),
                "dataSize": obj.get("dataSize"),
                "fileSize": obj.get("fileSize"),
                "lastBuildTimeS": obj.get("lastBuildTimeS"),
                "numberOfPendingTasks": obj.get("numberOfPendingTasks"),
                "pendingTask": obj.get("pendingTask"),
                "primary": obj.get("primary"),
                "replicas": obj.get("replicas"),
                "virtual": obj.get("virtual"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from enum import Enum
from json import loads
from typing import Self


class TriggerType(str, Enum):
    """
    Task trigger, describing when a task should run.  - `onDemand`.   Manually trigger the task with the `/run` endpoint.  - `schedule`.   Regularly trigger the task on a `cron` schedule.  - `subscription`.   Trigger the task after an event is received, such as, a webhook.  - `streaming`.   Run the task continuously.
    """

    """
    allowed enum values
    """
    ONDEMAND = "onDemand"
    SCHEDULE = "schedule"
    SUBSCRIPTION = "subscription"
    STREAMING = "streaming"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of TriggerType from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.0.0b26"



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from re import match
from typing import Annotated, Any, ClassVar, Dict, List, Optional, Self

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictBool,
    StrictInt,
    StrictStr,
    field_validator,
)

from algoliasearch.search.models.exhaustive import Exhaustive
from algoliasearch.search.models.facets_stats import FacetsStats
from algoliasearch.search.models.hit import Hit
from algoliasearch.search.models.redirect import Redirect
from algoliasearch.search.models.rendering_content import RenderingContent


class SearchResponse(BaseModel):
    """
    SearchResponse
    """

    ab_test_id: Optional[StrictInt] = Field(
        default=None,
        description="A/B test ID. This is only included in the response for indices that are part of an A/B test.",
        alias="abTestID",
    )
    ab_test_variant_id: Optional[Annotated[int, Field(strict=True, ge=1)]] = Field(
        default=None,
        description="Variant ID. This is only included in the response for indices that are part of an A/B test.",
        alias="abTestVariantID",
    )
    around_lat_lng: Optional[Annotated[str, Field(strict=True)]] = Field(
        default=None,
        description="Computed geographical location.",
        alias="aroundLatLng",
    )
    automatic_radius: Optional[StrictStr] = Field(
        default=None,
        description="Distance from a central coordinate provided by `aroundLatLng`.",
        alias="automaticRadius",
    )
    exhaustive: Optional[Exhaustive] = None
    exhaustive_facets_count: Optional[StrictBool] = Field(
        default=None,
        description="See the `facetsCount` field of the `exhaustive` object in the response.",
        alias="exhaustiveFacetsCount",
    )
    exhaustive_nb_hits: Optional[StrictBool] = Field(
        default=None,
        description="See the `nbHits` field of the `exhaustive` object in the response.",
        alias="exhaustiveNbHits",
    )
    exhaustive_typo: Optional[StrictBool] = Field(
        default=None,
        description="See the `typo` field of the `exhaustive` object in the response.",
        alias="exhaustiveTypo",
    )
    facets: Optional[Dict[str, Dict[str, StrictInt]]] = Field(
        default=None, description="Facet counts."
    )
    facets_stats: Optional[Dict[str, FacetsStats]] = Field(
        default=None, description="Statistics for numerical facets."
    )
    index: Optional[StrictStr] = Field(
        default=None, description="Index name used for the query."
    )
    index_used: Optional[StrictStr] = Field(
        default=None,
        description="Index name used for the query. During A/B testing, the targeted index isn't always the index used by the query.",
        alias="indexUsed",
    )
    message: Optional[StrictStr] = Field(
        default=None, description="Warnings about the query."
    )
    nb_sorted_hits: Optional[StrictInt] = Field(
        default=None,
        description="Number of hits selected and sorted by the relevant sort algorithm.",
        alias="nbSortedHits",
    )
    parsed_query: Optional[StrictStr] = Field(
        default=None,
        description="Post-[normalization](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/handling-natural-languages-nlp/#what-does-normalization-mean) query string that will be searched.",
        alias="parsedQuery",
    )
    processing_time_ms: StrictInt = Field(
        description="Time the server took to process the request, in milliseconds.",
        alias="processingTimeMS",
    )
    processing_timings_ms: Optional[Dict[str, Any]] = Field(
        default=None,
        description="Experimental. List of processing steps and their times, in milliseconds. You can use this list to investigate performance issues.",
        alias="processingTimingsMS",
    )
    query_after_removal: Optional[StrictStr] = Field(
        default=None,
        description="Markup text indicating which parts of the original query have been removed to retrieve a non-empty result set.",
        alias="queryAfterRemoval",
    )
    redirect: Optional[Redirect] = None
    rendering_content: Optional[RenderingContent] = Field(
        default=None, alias="renderingContent"
    )
    server_time_ms: Optional[StrictInt] = Field(
        default=None,
        description="Time the server took to process the request, in milliseconds.",
        alias="serverTimeMS",
    )
    server_used: Optional[StrictStr] = Field(
        default=None,
        description="Host name of the server that processed the request.",
        alias="serverUsed",
    )
    user_data: Optional[Dict[str, Any]] = Field(
        default=None,
        description="An object with custom data.  You can store up to 32kB as custom data. ",
        alias="userData",
    )
    query_id: Optional[StrictStr] = Field(
        default=None,
        description="Unique identifier for the query. This is used for [click analytics](https://www.algolia.com/doc/guides/analytics/click-analytics/).",
        alias="queryID",
    )
    page: Annotated[int, Field(strict=True, ge=0)] = Field(
        description="Page of search results to retrieve."
    )
    nb_hits: StrictInt = Field(description="Number of results (hits).", alias="nbHits")
    nb_pages: StrictInt = Field(
        description="Number of pages of results.", alias="nbPages"
    )
    hits_per_page: Annotated[int, Field(le=1000, strict=True, ge=1)] = Field(
        description="Number of hits per page.", alias="hitsPerPage"
    )
    hits: List[Hit] = Field(
        description="Search results (hits).  Hits are records from your index that match the search criteria, augmented with additional attributes, such as, for highlighting. "
    )
    query: StrictStr = Field(description="Search query.")
    params: StrictStr = Field(
        description="URL-encoded string of all search parameters."
    )
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = [
        "abTestID",
        "abTestVariantID",
        "aroundLatLng",
        "automaticRadius",
        "exhaustive",
        "exhaustiveFacetsCount",
        "exhaustiveNbHits",
        "exhaustiveTypo",
        "facets",
        "facets_stats",
        "index",
        "indexUsed",
        "message",
        "nbSortedHits",
        "parsedQuery",
        "processingTimeMS",
        "processingTimingsMS",
        "queryAfterRemoval",
        "redirect",
        "renderingContent",
        "serverTimeMS",
        "serverUsed",
        "userData",
        "queryID",
        "page",
        "nbHits",
        "nbPages",
        "hitsPerPage",
        "hits",
        "query",
        "params",
    ]

    @field_validator("around_lat_lng")
    def around_lat_lng_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not match(r"^(-?\d+(\.\d+)?),\s*(-?\d+(\.\d+)?)$", value):
            raise ValueError(
                r"must validate the regular expression /^(-?\d+(\.\d+)?),\s*(-?\d+(\.\d+)?)$/"
            )
        return value

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SearchResponse from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "additional_properties",
            },
            exclude_none=True,
        )
        if self.exhaustive:
            _dict["exhaustive"] = self.exhaustive.to_dict()
        _field_dict = {}
        if self.facets_stats:
            for _key in self.facets_stats:
                if self.facets_stats[_key]:
                    _field_dict[_key] = self.facets_stats[_key].to_dict()
            _dict["facets_stats"] = _field_dict
        if self.redirect:
            _dict["redirect"] = self.redirect.to_dict()
        if self.rendering_content:
            _dict["renderingContent"] = self.rendering_content.to_dict()
        _items = []
        if self.hits:
            for _item in self.hits:
                if _item:
                    _items.append(_item.to_dict())
            _dict["hits"] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of SearchResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "abTestID": obj.get("abTestID"),
                "abTestVariantID": obj.get("abTestVariantID"),
                "aroundLatLng": obj.get("aroundLatLng"),
                "automaticRadius": obj.get("automaticRadius"),
                "exhaustive": (
                    Exhaustive.from_dict(obj.get("exhaustive"))
                    if obj.get("exhaustive") is not None
                    else None
                ),
                "exhaustiveFacetsCount": obj.get("exhaustiveFacetsCount"),
                "exhaustiveNbHits": obj.get("exhaustiveNbHits"),
                "exhaustiveTypo": obj.get("exhaustiveTypo"),
                "facets": obj.get("facets"),
                "facets_stats": (
                    dict(
                        (_k, FacetsStats.from_dict(_v))
                        for _k, _v in obj.get("facets_stats").items()
                    )
                    if obj.get("facets_stats") is not None
                    else None
                ),
                "index": obj.get("index"),
                "indexUsed": obj.get("indexUsed"),
                "message": obj.get("message"),
                "nbSortedHits": obj.get("nbSortedHits"),
                "parsedQuery": obj.get("parsedQuery"),
                "processingTimeMS": obj.get("processingTimeMS"),
                "processingTimingsMS": obj.get("processingTimingsMS"),
                "queryAfterRemoval": obj.get("queryAfterRemoval"),
                "redirect": (
                    Redirect.from_dict(obj.get("redirect"))
                    if obj.get("redirect") is not None
                    else None
                ),
                "renderingContent": (
                    RenderingContent.from_dict(obj.get("renderingContent"))
                    if obj.get("renderingContent") is not None
                    else None
                ),
                "serverTimeMS": obj.get("serverTimeMS"),
                "serverUsed": obj.get("serverUsed"),
                "userData": obj.get("userData"),
                "queryID": obj.get("queryID"),
                "page": obj.get("page"),
                "nbHits": obj.get("nbHits"),
                "nbPages": obj.get("nbPages"),
                "hitsPerPage": obj.get("hitsPerPage"),
                "hits": (
                    [Hit.from_dict(_item) for _item in obj.get("hits")]
                    if obj.get("hits") is not None
                    else None
                ),
                "query": obj.get("query"),
                "params": obj.get("params"),
            }
        )
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from typing import Any, Dict, Self

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr


class SaveSynonymResponse(BaseModel):
    """
    SaveSynonymResponse
    """

    task_id: StrictInt = Field(
        description="Unique identifier of a task.  A successful API response means that a task was added to a queue. It might not run immediately. You can check the task's progress with the [`task` operation](#tag/Indices/operation/getTask) and this `taskID`. ",
        alias="taskID",
    )
    updated_at: StrictStr = Field(
        description="Date and time when the object was updated, in RFC 3339 format.",
        alias="updatedAt",
    )
    id: StrictStr = Field(description="Unique identifier of a synonym object.")

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SaveSynonymResponse from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of SaveSynonymResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "taskID": obj.get("taskID"),
                "updatedAt": obj.get("updatedAt"),
                "id": obj.get("id"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from enum import Enum
from json import loads
from typing import Self


class SupportedLanguage(str, Enum):
    """
    ISO code for a supported language.
    """

    """
    allowed enum values
    """
    AF = "af"
    AR = "ar"
    AZ = "az"
    BG = "bg"
    BN = "bn"
    CA = "ca"
    CS = "cs"
    CY = "cy"
    DA = "da"
    DE = "de"
    EL = "el"
    EN = "en"
    EO = "eo"
    ES = "es"
    ET = "et"
    EU = "eu"
    FA = "fa"
    FI = "fi"
    FO = "fo"
    FR = "fr"
    GA = "ga"
    GL = "gl"
    HE = "he"
    HI = "hi"
    HU = "hu"
    HY = "hy"
    ID = "id"
    IS = "is"
    IT = "it"
    JA = "ja"
    KA = "ka"
    KK = "kk"
    KO = "ko"
    KU = "ku"
    KY = "ky"
    LT = "lt"
    LV = "lv"
    MI = "mi"
    MN = "mn"
    MR = "mr"
    MS = "ms"
    MT = "mt"
    NB = "nb"
    NL = "nl"
    NO = "no"
    NS = "ns"
    PL = "pl"
    PS = "ps"
    PT = "pt"
    PT_MINUS_BR = "pt-br"
    QU = "qu"
    RO = "ro"
    RU = "ru"
    SK = "sk"
    SQ = "sq"
    SV = "sv"
    SW = "sw"
    TA = "ta"
    TE = "te"
    TH = "th"
    TL = "tl"
    TN = "tn"
    TR = "tr"
    TT = "tt"
    UK = "uk"
    UR = "ur"
    UZ = "uz"
    ZH = "zh"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SupportedLanguage from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from enum import Enum
from json import loads
from typing import Self


class Status(str, Enum):
    """
    A/B test status.  - `active`. The A/B test is live and search traffic is split between the two variants. - `stopped`. You stopped the A/B test. The A/B test data is still available for analysis. - `expired`. The A/B test was automatically stopped after reaching its end date. - `failed`. Creating the A/B test failed.
    """

    """
    allowed enum values
    """
    ACTIVE = "active"
    STOPPED = "stopped"
    EXPIRED = "expired"
    FAILED = "failed"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Status from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from typing import Annotated, Any, Dict, Optional, Self

from pydantic import BaseModel, ConfigDict, Field, StrictStr

from algoliasearch.ingestion.models.mapping_type_csv import MappingTypeCSV
from algoliasearch.ingestion.models.method_type import MethodType


class SourceCSV(BaseModel):
    """
    SourceCSV
    """

    url: StrictStr = Field(description="URL of the file.")
    unique_id_column: Optional[StrictStr] = Field(
        default=None,
        description="Name of a column that contains a unique ID which will be used as `objectID` in Algolia.",
        alias="uniqueIDColumn",
    )
    mapping: Optional[Dict[str, MappingTypeCSV]] = Field(
        default=None,
        description="Key-value pairs of column names and their expected types. ",
    )
    method: Optional[MethodType] = None
    delimiter: Optional[
        Annotated[str, Field(min_length=1, strict=True, max_length=1)]
    ] = Field(
        default=",",
        description="The character used to split the value on each line, default to a comma (\\r, \\n, 0xFFFD, and space are forbidden).",
    )

    model_config = ConfigDict(
        use_enum_values=True, populate_by_name=True, validate_assignment=True
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SourceCSV from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of SourceCSV from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "url": obj.get("url"),
                "uniqueIDColumn": obj.get("uniqueIDColumn"),
                "mapping": dict((_k, _v) for _k, _v in obj.get("mapping").items()),
                "method": obj.get("method"),
                "delimiter": obj.get("delimiter"),
            }
        )
        return _obj



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from enum import Enum
from json import loads
from typing import Self


class ExactOnSingleWordQuery(str, Enum):
    """
    Determines how the [Exact ranking criterion](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/override-search-engine-defaults/in-depth/adjust-exact-settings/#turn-off-exact-for-some-attributes) is computed when the search query has only one word.  - `attribute`.   The Exact ranking criterion is 1 if the query word and attribute value are the same.   For example, a search for \"road\" will match the value \"road\", but not \"road trip\".  - `none`.   The Exact ranking criterion is ignored on single-word searches.  - `word`.   The Exact ranking criterion is 1 if the query word is found in the attribute value.   The query word must have at least 3 characters and must not be a stop word.   Only exact matches will be highlighted,   partial and prefix matches won't.
    """

    """
    allowed enum values
    """
    ATTRIBUTE = "attribute"
    NONE = "none"
    WORD = "word"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ExactOnSingleWordQuery from a JSON string"""
        return cls(loads(json_str))
